@@ -61,7 +61,7 @@ solve(const char *equation)
         exit(1);
     }
     else {
-        mp_cast_to_string(&result, 10, 10, 9, 1, result_str, 1024);
+        mp_cast_to_string(&result, 10, 10, 9, 1, '.', result_str, 1024);
         printf("%s\n", result_str);
         exit(0);
     }
@@ -115,7 +115,7 @@ usage(const gchar *progname, gboolean show_application, gboolean show_gtk)
 }
 
 
-void
+static void
 get_options(int argc, char *argv[])
 {
     int i;
@@ -193,6 +193,7 @@ quit_cb(MathWindow *window)
     g_settings_set_enum(settings, "button-mode", math_buttons_get_mode(buttons));
     g_settings_set_string(settings, "source-currency", math_equation_get_source_currency(equation));
     g_settings_set_string(settings, "target-currency", math_equation_get_target_currency(equation));
+    g_settings_sync();
 
     currency_free_resources();
     gtk_main_quit();
@@ -212,6 +213,7 @@ main(int argc, char **argv)
 
     g_type_init();
 
+    setlocale(LC_ALL, "");
     bindtextdomain(GETTEXT_PACKAGE, LOCALE_DIR);
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
