@@ -97,7 +97,7 @@ registers_save(MathVariables *variables)
         MPNumber *value = val;
         char number[1024];
 
-        mp_cast_to_string(value, 10, 10, 50, TRUE, number, 1024);
+        mp_cast_to_string(value, 10, 10, 50, TRUE, '.', number, 1024);
         fprintf(f, "%s=%s\n", name, number);
     }
     fclose(f);
