@@ -20,7 +20,7 @@
 #include <stdio.h>
 
 #include "mp.h"
-#include "mp-internal.h"
+#include "mp-private.h"
 
 // FIXME: Make dynamic
 #define MAX_DIGITS 1000
@@ -45,10 +45,10 @@ mp_bitwise(const MPNumber *x, const MPNumber *y, int (*bitwise_operator)(int, in
     char text1[MAX_DIGITS], text2[MAX_DIGITS], text_out[MAX_DIGITS], text_out2[MAX_DIGITS];
     int offset1, offset2, offset_out;
 
-    mp_cast_to_string(x, 16, 0, 0, text1, MAX_DIGITS);
-    mp_cast_to_string(y, 16, 0, 0, text2, MAX_DIGITS);
-    offset1 = strlen(text1) - 1 - strlen("₁₆");
-    offset2 = strlen(text2) - 1 - strlen("₁₆");
+    mp_cast_to_string(x, 16, 16, 0, 0, '.', text1, MAX_DIGITS);
+    mp_cast_to_string(y, 16, 16, 0, 0, '.', text2, MAX_DIGITS);
+    offset1 = strlen(text1) - 1;
+    offset2 = strlen(text2) - 1;
     offset_out = wordlen / 4 - 1;
     if (offset_out <= 0) {
         offset_out = offset1 > offset2 ? offset1 : offset2;
@@ -73,8 +73,8 @@ mp_bitwise(const MPNumber *x, const MPNumber *y, int (*bitwise_operator)(int, in
         text_out[offset_out] = digits[bitwise_operator(v1, v2)];
     }
 
-    snprintf(text_out2, MAX_DIGITS, "%s₁₆", text_out);
-    mp_set_from_string(text_out2, z);
+    snprintf(text_out2, MAX_DIGITS, "%s", text_out);
+    mp_set_from_string(text_out2, 16, z);
 }
 
 
@@ -84,7 +84,7 @@ static int mp_bitwise_xor(int v1, int v2) { return v1 ^ v2; }
 static int mp_bitwise_not(int v1, int dummy) { return v1 ^ 0xF; }
 
 
-int
+bool
 mp_is_overflow (const MPNumber *x, int wordlen)
 {
     MPNumber tmp1, tmp2;
@@ -97,10 +97,10 @@ mp_is_overflow (const MPNumber *x, int wordlen)
 void
 mp_and(const MPNumber *x, const MPNumber *y, MPNumber *z)
 {
-    if (!mp_is_natural(x) || !mp_is_natural(y))
+    if (!mp_is_positive_integer(x) || !mp_is_positive_integer(y))
     {
         /* Translators: Error displayed when boolean AND attempted on non-integer values */
-        mperr(_("Boolean AND is only defined for natural numbers"));
+        mperr(_("Boolean AND is only defined for positive integers"));
     }
 
     mp_bitwise(x, y, mp_bitwise_and, z, 0);
@@ -110,10 +110,10 @@ mp_and(const MPNumber *x, const MPNumber *y, MPNumber *z)
 void
 mp_or(const MPNumber *x, const MPNumber *y, MPNumber *z)
 {
-    if (!mp_is_natural(x) || !mp_is_natural(y))
+    if (!mp_is_positive_integer(x) || !mp_is_positive_integer(y))
     {
         /* Translators: Error displayed when boolean OR attempted on non-integer values */
-        mperr(_("Boolean OR is only defined for natural numbers"));
+        mperr(_("Boolean OR is only defined for positive integers"));
     }
 
     mp_bitwise(x, y, mp_bitwise_or, z, 0);
@@ -123,10 +123,10 @@ mp_or(const MPNumber *x, const MPNumber *y, MPNumber *z)
 void
 mp_xor(const MPNumber *x, const MPNumber *y, MPNumber *z)
 {
-    if (!mp_is_natural(x) || !mp_is_natural(y))
+    if (!mp_is_positive_integer(x) || !mp_is_positive_integer(y))
     {
         /* Translators: Error displayed when boolean XOR attempted on non-integer values */
-        mperr(_("Boolean XOR is only defined for natural numbers"));
+        mperr(_("Boolean XOR is only defined for positive integers"));
     }
 
     mp_bitwise(x, y, mp_bitwise_xor, z, 0);
@@ -138,10 +138,10 @@ mp_not(const MPNumber *x, int wordlen, MPNumber *z)
 {
     MPNumber temp;
 
-    if (!mp_is_natural(x))
+    if (!mp_is_positive_integer(x))
     {
         /* Translators: Error displayed when boolean XOR attempted on non-integer values */
-        mperr(_("Boolean NOT is only defined for natural numbers"));
+        mperr(_("Boolean NOT is only defined for positive integers"));
     }
 
     mp_set_from_integer(0, &temp);
@@ -156,11 +156,11 @@ mp_mask(const MPNumber *x, int wordlen, MPNumber *z)
     size_t len, offset;
 
     /* Convert to a hexadecimal string and use last characters */
-    mp_cast_to_string(x, 16, 0, 0, text, MAX_DIGITS);
-    len = strlen(text) - strlen("₁₆");
+    mp_cast_to_string(x, 16, 16, 0, 0, '.', text, MAX_DIGITS);
+    len = strlen(text);
     offset = wordlen / 4;
     offset = len > offset ? len - offset: 0;
-    mp_set_from_string(text + offset, z);
+    mp_set_from_string(text + offset, 16, z);
 }
 
 
@@ -185,7 +185,7 @@ mp_shift(const MPNumber *x, int count, MPNumber *z)
         for (i = 0; i < -count; i++)
             multiplier *= 2;
         mp_divide_integer(x, multiplier, &temp);
-        mp_integer_component(&temp, z);
+        mp_floor(&temp, z);
     }
 }
 
