@@ -18,12 +18,14 @@
  */
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+#include <langinfo.h>
 
 #include "mp.h"
-#include "mp-internal.h"
+#include "mp-private.h"
 
 void
 mp_set_from_mp(const MPNumber *x, MPNumber *z)
@@ -39,7 +41,7 @@ mp_set_from_float(float rx, MPNumber *z)
     int i, k, ib, ie, tp;
     float rj;
 
-    memset(z, 0, sizeof(MPNumber));
+    mp_set_from_integer(0, z);
 
     /* CHECK SIGN */
     if (rx < 0.0f) {
@@ -112,7 +114,7 @@ mp_set_from_double(double dx, MPNumber *z)
     int i, k, ib, ie, tp;
     double dj;
 
-    memset(z, 0, sizeof(MPNumber));
+    mp_set_from_integer(0, z);
 
     /* CHECK SIGN */
     if (dx < 0.0)  {
@@ -195,8 +197,8 @@ mp_set_from_integer(int64_t x, MPNumber *z)
 
     while (x != 0) {
         z->fraction[z->exponent] = x % MP_BASE;
-        x = x / MP_BASE;
         z->exponent++;
+        x /= MP_BASE;
     }
     for (i = 0; i < z->exponent / 2; i++) {
         int t = z->fraction[i];
@@ -211,7 +213,7 @@ mp_set_from_unsigned_integer(uint64_t x, MPNumber *z)
 {
     int i;
 
-    memset(z, 0, sizeof(MPNumber));
+    mp_set_from_integer(0, z);
 
     if (x == 0) {
         z->sign = 0;
@@ -257,24 +259,28 @@ mp_set_from_fraction(int64_t numerator, int64_t denominator, MPNumber *z)
 void
 mp_set_from_polar(const MPNumber *r, MPAngleUnit unit, const MPNumber *theta, MPNumber *z)
 {
-    MPNumber sin_theta, cos_theta;
-    
-    mp_sin(theta, unit, &sin_theta);
-    mp_cos(theta, unit, &cos_theta);
-    mp_set_from_complex(&cos_theta, &sin_theta, z);
+    MPNumber x, y;
+
+    mp_cos(theta, unit, &x);
+    mp_multiply(&x, r, &x);
+    mp_sin(theta, unit, &y);
+    mp_multiply(&y, r, &y);
+    mp_set_from_complex(&x, &y, z);
 }
 
 
 void
 mp_set_from_complex(const MPNumber *x, const MPNumber *y, MPNumber *z)
 {
-    //MPNumber i;
+    /* NOTE: Do imaginary component first as z may be x or y */
+    z->im_sign = y->sign;
+    z->im_exponent = y->exponent;
+    memcpy(z->im_fraction, y->fraction, sizeof(int) * MP_SIZE);
 
-    // FIXME: There is some corruption here as i is currently defined to zero but the result does not work
-    //mp_get_i(&i);
-    //mp_multiply(y, &i, z);
-    //mp_add(x, z, z);
-    mp_set_from_mp(x, z);
+    z->sign = x->sign;
+    z->exponent = x->exponent;
+    if (z != x)
+        memcpy(z->fraction, x->fraction, sizeof(int) * MP_SIZE);
 }
 
 
@@ -288,90 +294,82 @@ mp_set_from_random(MPNumber *z)
 int64_t
 mp_cast_to_int(const MPNumber *x)
 {
-    int i, j;
-    int64_t ret_val = 0;
-
-    /* RETURN 0 IF X = 0 OR IF NUMBER FRACTION */
+    int i;
+    int64_t z = 0, v;
+  
+    /* |x| <= 1 */
     if (x->sign == 0 || x->exponent <= 0)
         return 0;
 
+    /* Multiply digits together */
     for (i = 0; i < x->exponent; i++) {
-        int izs;
-        izs = ret_val;
-        ret_val = MP_BASE * ret_val;
-        if (i < MP_T)
-            ret_val += x->fraction[i];
+        int64_t t;
 
-        /* CHECK FOR SIGNS OF INTEGER OVERFLOW */
-        if (ret_val <= 0 || ret_val <= izs)
+        t = z;
+        z = z * MP_BASE + x->fraction[i];
+      
+        /* Check for overflow */
+        if (z <= t)
             return 0;
     }
 
-    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY HAVE OCCURRED). */
-    j = ret_val;
+    /* Validate result */
+    v = z;
     for (i = x->exponent - 1; i >= 0; i--) {
-        int j1, kx;
+        int64_t digit;
 
-        j1 = j / MP_BASE;
-        kx = 0;
-        if (i < MP_T)
-            kx = x->fraction[i];
-        if (kx != j - MP_BASE * j1)
+        /* Get last digit */
+        digit = v - (v / MP_BASE) * MP_BASE;
+        if (x->fraction[i] != digit)
             return 0;
-        j = j1;
+
+        v /= MP_BASE;
     }
-    if (j != 0)
+    if (v != 0)
         return 0;
 
-    /* RESULT CORRECT SO RESTORE SIGN AND RETURN */
-    return x->sign * ret_val;
-
-    /* Old comment about returning zero: */
-    /*  HERE OVERFLOW OCCURRED (OR X WAS UNNORMALIZED), SO
-     *  RETURN ZERO.
-     */
+    return x->sign * z;
 }
 
 
 uint64_t
 mp_cast_to_unsigned_int(const MPNumber *x)
 {
-    int i, j;
-    uint64_t ret_val = 0;
-
-    /* RETURN 0 IF X <= 0 OR IF NUMBER FRACTION */
+    int i;
+    uint64_t z = 0, v;
+  
+    /* x <= 1 */
     if (x->sign <= 0 || x->exponent <= 0)
         return 0;
 
+    /* Multiply digits together */
     for (i = 0; i < x->exponent; i++) {
-        int izs;
-        izs = ret_val;
-        ret_val = MP_BASE * ret_val;
-        if (i < MP_T)
-            ret_val += x->fraction[i];
+        uint64_t t;
 
-        /* CHECK FOR SIGNS OF INTEGER OVERFLOW */
-        if (ret_val <= 0 || ret_val <= izs)
+        t = z;
+        z = z * MP_BASE + x->fraction[i];
+      
+        /* Check for overflow */
+        if (z <= t)
             return 0;
     }
 
-    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY HAVE OCCURRED). */
-    j = ret_val;
+    /* Validate result */
+    v = z;
     for (i = x->exponent - 1; i >= 0; i--) {
-        int j1, kx;
+        uint64_t digit;
 
-        j1 = j / MP_BASE;
-        kx = 0;
-        if (i < MP_T)
-            kx = x->fraction[i];
-        if (kx != j - MP_BASE * j1)
+        /* Get last digit */
+        digit = v - (v / MP_BASE) * MP_BASE;
+        if (x->fraction[i] != digit)
             return 0;
-        j = j1;
+
+        v /= MP_BASE;
     }
-    if (j != 0)
+    if (v != 0)
         return 0;
 
-    return ret_val;
+    return z;
 }
 
 
@@ -511,7 +509,7 @@ mp_cast_to_double(const MPNumber *x)
 
 
 static void
-mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zeroes, bool force_sign, GString *string)
+mp_cast_to_string_real(const MPNumber *x, int default_base, int base, int accuracy, bool trim_zeroes, gunichar radix, bool force_sign, GString *string)
 {
     static char digits[] = "0123456789ABCDEF";
     MPNumber number, integer_component, fractional_component, temp;
@@ -530,28 +528,29 @@ mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zero
     mp_add(&number, &temp, &number);
 
     /* Split into integer and fractional component */
-    mp_integer_component(&number, &integer_component);
+    mp_floor(&number, &integer_component);
     mp_fractional_component(&number, &fractional_component);
 
     /* Write out the integer component least significant digit to most */
     mp_set_from_mp(&integer_component, &temp);
     do {
         MPNumber t, t2, t3;
+        int64_t d;
 
         mp_divide_integer(&temp, base, &t);
-        mp_integer_component(&t, &t);
+        mp_floor(&t, &t);
         mp_multiply_integer(&t, base, &t2);
 
         mp_subtract(&temp, &t2, &t3);
-        mp_integer_component(&t3, &t3);
 
-        g_string_prepend_c(string, digits[mp_cast_to_int(&t3)]);
+        d = mp_cast_to_int(&t3);
+        g_string_prepend_c(string, d < 16 ? digits[d] : '?');
 
         mp_set_from_mp(&t, &temp);
     } while (!mp_is_zero(&temp));
 
     last_non_zero = string->len;
-    g_string_append_c(string, '.');
+    g_string_append_unichar(string, radix);
 
     /* Write out the fractional component */
     mp_set_from_mp(&fractional_component, &temp);
@@ -560,7 +559,7 @@ mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zero
         MPNumber digit;
 
         mp_multiply_integer(&temp, base, &temp);
-        mp_integer_component(&temp, &digit);
+        mp_floor(&temp, &digit);
         d = mp_cast_to_int(&digit);
 
         g_string_append_c(string, digits[d]);
@@ -575,55 +574,60 @@ mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zero
         g_string_truncate(string, last_non_zero);
 
     /* Add sign on non-zero values */
-    if (strcmp(string->str, "0") != 0) {
+    if (strcmp(string->str, "0") != 0 || force_sign) {
         if (mp_is_negative(x))
             g_string_prepend(string, "−");
         else if (force_sign)
             g_string_prepend(string, "+");
     }
 
-    switch(base)
-    {
-    case 2:
-        g_string_append(string, "₂");
-        break;
-    case 8:
-        g_string_append(string, "₈");
-        break;
-    default:
-    case 10:
-        break;
-    case 16:
-        g_string_append(string, "₁₆");
-        break;
+    /* Append base suffix if not in default base */
+    if (base != default_base) {
+        const char *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
+        int multiplier = 1;
+        int b = base;
+
+        while (base / multiplier != 0)
+            multiplier *= 10;
+        while (multiplier != 1) {
+            int d;
+            multiplier /= 10;
+            d = b / multiplier;
+            g_string_append(string, digits[d]);
+            b -= d * multiplier;
+        }
     }
 }
 
 
 void
-mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, char *buffer, int buffer_length)
+mp_cast_to_string(const MPNumber *x, int default_base, int base, int accuracy, bool trim_zeroes, gunichar radix, char *buffer, int buffer_length)
 {
-    MPNumber x_real, x_im;
     GString *string;
-    
+    MPNumber x_real;
+
     string = g_string_sized_new(buffer_length);
-    
+
     mp_real_component(x, &x_real);
-    mp_imaginary_component(x, &x_im);
-    
-    mp_cast_to_string_real(&x_real, base, accuracy, trim_zeroes, FALSE, string);
+    mp_cast_to_string_real(&x_real, default_base, base, accuracy, trim_zeroes, radix, FALSE, string);
     if (mp_is_complex(x)) {
         GString *s;
         gboolean force_sign = TRUE;
-        
+        MPNumber x_im;
+
+        mp_imaginary_component(x, &x_im);
+
         if (strcmp(string->str, "0") == 0) {
             g_string_assign(string, "");
-            force_sign = FALSE;
+            force_sign = false;
         }
 
         s = g_string_sized_new(buffer_length);
-        mp_cast_to_string_real(&x_im, 10, accuracy, trim_zeroes, force_sign, s);
-        if (strcmp(s->str, "1") == 0) {
+        mp_cast_to_string_real(&x_im, default_base, 10, accuracy, trim_zeroes, radix, force_sign, s);
+        if (strcmp(s->str, "0") == 0 || strcmp(s->str, "+0") == 0 || strcmp(s->str, "−0") == 0) {
+            /* Ignore */
+        }
+        else if (strcmp(s->str, "1") == 0) {
             g_string_append(string, "i");
         }
         else if (strcmp(s->str, "+1") == 0) {
@@ -633,7 +637,11 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, c
             g_string_append(string, "−i");
         }
         else {
-            g_string_append(string, s->str);
+            if (strcmp(s->str, "+0") == 0)
+                g_string_append(string, "+");
+            else if (strcmp(s->str, "0") != 0)
+                g_string_append(string, s->str);
+
             g_string_append(string, "i");
         }
         g_string_free(s, TRUE);
@@ -645,11 +653,76 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, c
 }
 
 
+void
+mp_cast_to_exponential_string(const MPNumber *x, int default_base, int base_, int max_digits, bool trim_zeroes, gunichar radix, bool eng_format, char *buffer, int buffer_length)
+{
+    char fixed[1024], *c;
+    MPNumber t, z, base, base3, base10, base10inv, mantissa;
+    int exponent = 0;
+    GString *string;
+    const char *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
+
+    string = g_string_sized_new(buffer_length);
+
+    mp_abs(x, &z);
+    if (mp_is_negative(x))
+        g_string_append(string, "−");
+    mp_set_from_mp(&z, &mantissa);
+
+    mp_set_from_integer(base_, &base);
+    mp_xpowy_integer(&base, 3, &base3);
+    mp_xpowy_integer(&base, 10, &base10);
+    mp_set_from_integer(1, &t);
+    mp_divide(&t, &base10, &base10inv);
+
+    if (!mp_is_zero(&mantissa)) {
+        while (!eng_format && mp_is_greater_equal(&mantissa, &base10)) {
+            exponent += 10;
+            mp_multiply(&mantissa, &base10inv, &mantissa);
+        }
+
+        while ((!eng_format &&  mp_is_greater_equal(&mantissa, &base)) ||
+                (eng_format && (mp_is_greater_equal(&mantissa, &base3) || exponent % 3 != 0))) {
+            exponent += 1;
+            mp_divide(&mantissa, &base, &mantissa);
+        }
+
+        while (!eng_format && mp_is_less_than(&mantissa, &base10inv)) {
+            exponent -= 10;
+            mp_multiply(&mantissa, &base10, &mantissa);
+        }
+
+        mp_set_from_integer(1, &t);
+        while (mp_is_less_than(&mantissa, &t) || (eng_format && exponent % 3 != 0)) {
+            exponent -= 1;
+            mp_multiply(&mantissa, &base, &mantissa);
+        }
+    }
+
+    mp_cast_to_string(&mantissa, default_base, base_, max_digits, trim_zeroes, radix, fixed, 1024);
+    g_string_append(string, fixed);
+    if (exponent != 0) {
+        g_string_append_printf(string, "×10"); // FIXME: Use the current base
+        if (exponent < 0) {
+            exponent = -exponent;
+            g_string_append(string, "⁻");
+        }
+        snprintf(fixed, 1024, "%d", exponent);
+        for (c = fixed; *c; c++)
+            g_string_append(string, super_digits[*c - '0']);
+    }
+
+    strncpy(buffer, string->str, buffer_length);
+    g_string_free(string, TRUE);
+}
+
+
 static int
 char_val(char **c, int base)
 {
     int i, j, value, offset;
     const char *digits[][10] = {{"٠", "١", "٢", "٣", "٤", "٥", "٦", "٧", "٨", "٩"},
+                                {"〇", "〡", "〢", "〣", "〤", "〥", "〦", "〧", "〨", "〩"},
                                 {"۰", "۱", "۲", "۳", "۴", "۵", "۶", "۷", "۸", "۹"},
                                 {"߀", "߁", "߂", "߃", "߄", "߅", "߆", "߇", "߈", "߉"},
                                 {"०", "१", "२", "३", "४", "५", "६", "७", "८", "९"},
@@ -725,8 +798,38 @@ ends_with(const char *start, const char *end, const char *word)
 }
 
 
+// FIXME: Doesn't handle errors well (e.g. trailing space)
+static bool
+set_from_sexagesimal(const char *str, int length, MPNumber *z)
+{
+    int degrees = 0, minutes = 0;
+    char seconds[length+1];
+    MPNumber t;
+    int n_matched;
+
+    seconds[0] = '\0';
+    n_matched = sscanf(str, "%d°%d'%s\"", &degrees, &minutes, seconds);
+
+    if (n_matched < 1)
+        return true;
+    mp_set_from_integer(degrees, z);
+    if (n_matched > 1) {
+        mp_set_from_integer(minutes, &t);
+        mp_divide_integer(&t, 60, &t);
+        mp_add(z, &t, z);
+    }
+    if (n_matched > 2) {
+        mp_set_from_string(seconds, 10, &t);
+        mp_divide_integer(&t, 3600, &t);
+        mp_add(z, &t, z);
+    }
+
+    return false;
+}
+
+
 bool
-mp_set_from_string(const char *str, MPNumber *z)
+mp_set_from_string(const char *str, int default_base, MPNumber *z)
 {
     int i, base, negate = 0, multiplier = 0, base_multiplier = 1;
     const char *c, *end;
@@ -736,6 +839,15 @@ mp_set_from_string(const char *str, MPNumber *z)
     const char *fractions[]     = {"½", "⅓", "⅔", "¼", "¾", "⅕", "⅖", "⅗", "⅘", "⅙", "⅚", "⅛", "⅜", "⅝", "⅞", NULL};
     int numerators[]            = { 1,   1,   2,   1,   3,   1,   2,   3,   4,   1,   5,   1,   3,   5,   7};
     int denominators[]          = { 2,   3,   3,   4,   4,   5,   5,   5,   5,   6,   6,   8,   8,   8,   8};
+    static const char *tsep = NULL;
+
+    if (tsep == NULL) {
+        tsep = nl_langinfo(THOUSEP);
+        tsep = *tsep ? g_locale_to_utf8(tsep, -1, NULL, NULL, NULL) : tsep;
+    }
+
+    if (strstr(str, "°"))
+        return set_from_sexagesimal(str, strlen(str), z);
 
     /* Find the base */
     end = str;
@@ -755,7 +867,7 @@ mp_set_from_string(const char *str, MPNumber *z)
             break;
     }
     if (base_multiplier == 1)
-        base = 10;
+        base = default_base;
 
     /* Check if this has a sign */
     c = str;
@@ -776,6 +888,9 @@ mp_set_from_string(const char *str, MPNumber *z)
             return true;
         mp_multiply_integer(z, base, z);
         mp_add_integer(z, i, z);
+
+        if (*tsep && strncmp(c, tsep, strlen(tsep)) == 0)
+            c += strlen(tsep);
     }
 
     /* Look for fraction characters, e.g. ⅚ */
